/*
 * Copyright (C) 1997 Wolfgang Moser
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * This Code based upon an AVL-Tree published in a german computer
 * magazine, c't, Issue 1/92, pages 174 ff.
 */

/*
 * This is the basic code for a node of an AVL-tree.
 * It is NOT instantiatable, because it has got no fields
 * holding some elements. The user (probably you) has to
 * create his own node, derived from PathAVLnode, with
 * additionally two added functions GT (greater than) and
 * NE (not equal). See the file tpavlnde.h for details and
 * as an example.
 *
 * The class PathAVLtree is declared as friend, because of
 * the better performance for direct assignements to the
 * successor- and balance-fields. A second advantage is the
 * better protection (!!!) of the class methods, because
 * I can declare them private, protecting them against
 * overloading.
 */

#ifndef _Path_AVL_Node_
#define _Path_AVL_Node_

#include "central.h"
#include <stddef.h>

class PathAVLnode{
   public:
      enum nodeside {LEFT=0, RIGHT=1}; // 0 und 1 ==> !-Operation mglich

      PathAVLnode   *getNode           (const nodeside Side)const;
      int            getBal            (void)const;
   protected:
      PathAVLnode(void);
      ~PathAVLnode(void);

   private:
         // direkten Zugriff auf Komponenten fr PathAVLtree ermglichen
      friend class PathAVLtree;

      static void    rotate            (const nodeside Side,
                                        PathAVLnode *&Father);
      static void    rebalanceAdded    (const nodeside Side,
                                        PathAVLnode *&Father);
      static int     rebalanceRemoved  (const nodeside Side,
                                        PathAVLnode *&Father);

      PathAVLnode   *Node[2];    // linker und rechter Knoten
      signed int     Balance:2;  // Bitfeld fr den Balancefaktor
         // signed-Angabe notwendig, da fr Bitfelder nicht
         // definiert ist, ob diese defaultmig signed oder
         // unsigned sind (Stroustroup, R9.6, Seite 605).
         // "implementationsabhngig", kann also jeder Compiler
         //                            machen wie er will

   };

inline PathAVLnode *PathAVLnode::getNode(const nodeside Side)const{
   return Node[Side];
   }

inline int PathAVLnode::getBal(void)const{
   return Balance;
   }

inline PathAVLnode::PathAVLnode(void){
   Node[0]=Node[1]=NULL;
   Balance=0;                 // Hoffe, das stimmt so
   }

inline PathAVLnode::~PathAVLnode(void){
   // nichts machen, sollte daher niemals direkt gerufen werden
   }

#endif
