/*
 * Copyright (C) 1997 Wolfgang Moser
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * This Code based upon an AVL-Tree published in a german computer
 * magazine, c't, Issue 1/92, pages 174 ff.
 *
 * I added a simple iterator-method for later use by iterator-objects.
 */

/*
 * This is the basic AVLtree-class, which implements all the basic
 * methods independent from any typed nodes. You can not instantiate
 * any objects of this class, because the constructor and desctrutor
 * are protected. You have to use the template-wrapper-class
 * TPathAVLtree for creating concretaly AVLtrees.
 */

#include "pavltree.h"

const char *PathAVLtree::version=_Path_AVL_Tree_version_;

void PathAVLtree::destroy(TStack<PathAVLnode *> &Stack){
   PathAVLnode *son,*act;  // aktueller besuchter Knoten


   if(Wurzel==NULL) return;
   act=Wurzel;
   // Post-Order-Traversierung (im Gegenesatz zur
   // In-Order-Traversierung der NextIter-Methode)
   for(;;){
      // weitesten links suchen (Stack fllen)
      while(act->Node[PathAVLnode::LEFT]!=NULL){
         Stack.push(act);
         act=act->Node[PathAVLnode::LEFT];
         }
      while(act->Node[PathAVLnode::RIGHT]==NULL){
            // aktueller Knoten kann gelscht werden
         DESTR(act);
         if(act==Wurzel){
            Wurzel=NULL;
            return;
            }
         son=act;
         act=Stack.pop();

         act->Node[act->Node[PathAVLnode::LEFT]==son
            ? PathAVLnode::LEFT
            : PathAVLnode::RIGHT
            ]=NULL;
         }
         // Der linke Sohn des act-Knotens ist nun NULL,
         // der rechte aber NICHT!
         // Also: einen nach rechts gehen und dann wieder den am
         // weitesten links stehenden suchen
      Stack.push(act);
      act=act->Node[PathAVLnode::RIGHT];
      }
   }

const PathAVLnode *PathAVLtree::nextIter(const PathAVLnode *search)const{
   PathAVLnode *succ=NULL, *act=Wurzel; // aktueller besuchter Knoten

   if(search==NULL){
         // weitesten links suchen (Initialisierung)
      while(act!=NULL){
         succ=act;
         act=act->Node[PathAVLnode::LEFT];
         }
      }
   else{
      while(act!=NULL && NEcmp(act,(PathAVLnode *)search)){
         if(GTcmp(act,(PathAVLnode *)search)){
               // sobald eine Verzweigung nach links stattgefunden
               // hat, existiert ein Nachfolger
            succ=act;
            act=act->Node[PathAVLnode::LEFT];
            }
         else act=act->Node[PathAVLnode::RIGHT];
         }
         // wenn Knoten gefunden act==Knoten,
         // wenn nicht gefunden: act==NULL

         // nchsten Knoten ansteuern
      if(act!=NULL && act->Node[PathAVLnode::RIGHT]!=NULL){
            // eins nach rechts und weitesten links suchen
         act=act->Node[PathAVLnode::RIGHT];
         while(act!=NULL){
            succ=act;
            act=act->Node[PathAVLnode::LEFT];
            }
         }
      // else
            // Wenn Knoten nicht gefunden oder keine Rechtsverzweigung
            // mehr mglich ist, den Nachfolger oder NULL liefern
      }
   return succ;
   }

   // 0, falls schon vorhanden!!
int PathAVLtree::add(PathAVLnode *NewNode){
   PathAVLnode          *act     =Wurzel; // aktueller besuchter Knoten
   PathAVLnode          *father  =NULL;   // Vater des aktuellen Knoten
   PathAVLnode          *critical=Wurzel; // kritischer Knoten
   PathAVLnode          *crfath  =NULL;   // Vater des kritischen Knoten

   if(NewNode==NULL) return 0;

   NewNode->Node[PathAVLnode::LEFT]=NewNode->Node[PathAVLnode::RIGHT]=NULL;

   while(act!=NULL && NEcmp(act,NewNode)){
      if(act->Balance){   // ungleich 0, also -1 oder +1
         crfath=father;
         critical=act;
         }
      father=act;
      gotoNext(act,NewNode);
      }
   if(act!=NULL) return 0; // Der Wert is bereits im Baum!!!
   else{
      if(Wurzel==NULL) Wurzel=NewNode; // Baum leer, also direkt einfgen
      else{
            // Neuen Knoten an der Vater hngen
         father->Node[(GTcmp(father,NewNode))
                           ? PathAVLnode::LEFT
                           : PathAVLnode::RIGHT
                        ]=NewNode;

         }
      if(critical!=NULL){
         act=critical;
         while(NEcmp(act,NewNode)){
            if(GTcmp(act,NewNode)){
               act->Balance--;
               act=act->Node[PathAVLnode::LEFT];
               }
            else{
               act->Balance++; // setBal(act->getBal()+1);
               act=act->Node[PathAVLnode::RIGHT];
               }
            }
         act=critical;
         if(act->Balance==-2){
            PathAVLnode::rebalanceAdded((GTcmp(act,NewNode))
                                          ? PathAVLnode::LEFT
                                          : PathAVLnode::RIGHT
                                          , act);
            if(crfath!=NULL)
               crfath->Node[(GTcmp(crfath,NewNode))
                                 ? PathAVLnode::LEFT
                                 : PathAVLnode::RIGHT
                              ]=act;
            else Wurzel=act;
            }
         }
      return 1;
      }
   }

   // NULL, falls nicht vorhanden, ansonsten der gelschte Knoten
PathAVLnode *PathAVLtree::remove(PathAVLnode *search,TStack<PathAVLnode *> &Stack){
   PathAVLnode            *act   =Wurzel; // aktueller besuchter Knoten
   PathAVLnode            *son   =NULL;   // Sohn des aktuellen Knoten
                                          // (zwischenzeitl. auch
                                          // Vater des remove)
   PathAVLnode            *remove=NULL;   // der zu entfernende Knoten
   PathAVLnode::nodeside   NSde  =PathAVLnode::LEFT;

   if(search==NULL) return NULL;

   Stack[0]=Wurzel;

   while(act!=NULL && NEcmp(act,search)){
      if(!act->Balance) Stack.empty();
      Stack.push(act);
      son=act;
      gotoNext(act,search);
      }
   if(act!=NULL){

      remove=act;    // ENTGEGEN nderung c't

      if(act->Node[PathAVLnode::LEFT]==NULL)
         son=act->Node[PathAVLnode::RIGHT];
      else{
         if(act->Node[PathAVLnode::RIGHT]==NULL)
            son=act->Node[PathAVLnode::LEFT];
         else{
            int RSPtr;
               // Den direkten Vorgnger des remove-Knotens suchen
            if(!act->Balance) Stack.empty();
               // Den Stapelplatz merken, auf dem mglicherweise remove gesichert wird.
            RSPtr=Stack.Ptr();
            Stack.push(act);
            act=act->Node[PathAVLnode::LEFT];
            while(act->Node[PathAVLnode::RIGHT]!=NULL){
               if(!act->Balance) Stack.empty();
               Stack.push(act);
               act=act->Node[PathAVLnode::RIGHT];
               }
            search=act; // den eigentlich zu entfernenden Knoten merken
               // den remove-Knoten ausketten, dafr den act-Knoten einfgen
               // alle Zeiger auf den remove-Knoten anpassen
            if(Stack[RSPtr]==remove) Stack[RSPtr]=search;
            if(son!=NULL)
               son->Node[(son->Node[PathAVLnode::LEFT]==remove)
                              ? PathAVLnode::LEFT
                              : PathAVLnode::RIGHT
                           ]=search;
            else{
               if(Stack[0]==Wurzel) Stack[0]=search;
               Wurzel=search;
               }

               // den alten Sohn des zu lschenden Knotens retten
            son=act->Node[PathAVLnode::LEFT];

               // Die Shne des zu lschenden Knotens an den neuen
               // Knoten hngen
            search->Node[PathAVLnode::LEFT] =remove->Node[PathAVLnode::LEFT];
            search->Node[PathAVLnode::RIGHT]=remove->Node[PathAVLnode::RIGHT];
               // Anpassung, falls der search.LEFT auf search zeigt (kann passieren)
            if(search->Node[PathAVLnode::LEFT]==search)
               search->Node[PathAVLnode::LEFT]=NULL;
               // Den korrekten Balancefaktor eintragen
            search->Balance=remove->Balance;
            }
         }
         // Die Shne des zu lschenden Knotens abhngen
      remove->Node[PathAVLnode::LEFT]=NULL;
      remove->Node[PathAVLnode::RIGHT]=NULL;

      while(!Stack.isEmpty()){
         act=Stack.pop();

         NSde=(GTcmp(search,act))
               ? PathAVLnode::RIGHT
               : PathAVLnode::LEFT;

         act->Node[NSde]=son;

         son=act;
         if(PathAVLnode::rebalanceRemoved(NSde,son)){
               // Alles schon fertig!
            if(!Stack.isEmpty()){
               act=Stack.top();
               if(GTcmp(search,act))
                  act->Node[PathAVLnode::RIGHT]=son;
               else
                  act->Node[PathAVLnode::LEFT]=son;
               }
            break;
            }
         }
      if(Stack.isEmpty() && Stack[0]==Wurzel) Wurzel=son;
      }
   return remove;
   }

   // NULL, falls nicht vorhanden, ansonsten der gefundene Knoten
const PathAVLnode *PathAVLtree::search(PathAVLnode * find)const{
   PathAVLnode *act     =Wurzel; // aktueller besuchter Knoten

   if(find==NULL) return NULL;

   while(act!=NULL && NEcmp(act,find)){
      gotoNext(act,find);
      }
   return act;
   }
