/*
 * Copyright (C) 1997 Wolfgang Moser
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * This Code based upon an AVL-Tree published in a german computer
 * magazine, c't, Issue 1/92, pages 174 ff.
 */

/*
 * this is the AVLtree-wrapper-class for instantiating concrete
 * (typed) AVL-trees. Storable objects have to been implemented
 * in a way, that they inherit from PathAVLnode and additionally
 * implement the methods GT and NE.
 * See pavlnode.h and tpavlnde.h for details.
 */

#ifndef _Typed_Path_AVL_Tree_
#define _Typed_Path_AVL_Tree_

#include "central.h"
#include "ststack.h"
#include "pavltree.h"
#include "tpavlnde.h"

template<class T, int size>
class TPathAVLtree:public PathAVLtree{
   public:
      TPathAVLtree(void);
      ~TPathAVLtree(void);

      void        empty    (void);
      const T    *nextIter (const T *)const;
         // 0, falls schon vorhanden!!
      int         add      (T *);
         // NULL, falls nicht vorhanden, ansonsten der gelschte Knoten
      T          *remove   (T *);
         // NULL, falls nicht vorhanden, ansonsten der gefundene Knoten
      const T    *search   (T *) const;

   protected:
      int         GTcmp(PathAVLnode *,PathAVLnode *)const;
      int         NEcmp(PathAVLnode *,PathAVLnode *)const;
      void        DESTR(PathAVLnode *)const;
   };

template<class T, int size>
inline TPathAVLtree<T,size>::TPathAVLtree(void)
   :PathAVLtree(){
   }

template<class T, int size>
inline TPathAVLtree<T,size>::~TPathAVLtree(void){
   empty();
   }

template<class T, int size>
inline void TPathAVLtree<T,size>::empty(void){
   STStack<PathAVLnode *,size> SS;
   PathAVLtree::destroy(SS);
   }

template<class T, int size>
inline const T *TPathAVLtree<T,size>::nextIter(const T *search)const{
   return (const T *)PathAVLtree::nextIter(search);
   }

   // 0, falls schon vorhanden!!
template<class T, int size>
inline int TPathAVLtree<T,size>::add(T *node){
   return PathAVLtree::add(node);
   }

   // NULL, falls nicht vorhanden, ansonsten der gelschte Knoten
template<class T, int size>
inline T *TPathAVLtree<T,size>::remove(T *search){
   STStack<PathAVLnode *,size> SS;
   return (T *)PathAVLtree::remove(search,SS);
   }

   // NULL, falls nicht vorhanden, ansonsten der gefundene Knoten
template<class T, int size>
inline const T *TPathAVLtree<T,size>::search(T *find) const{
   return (const T *)PathAVLtree::search(find);
   }

template<class T, int size>
inline int TPathAVLtree<T,size>::GTcmp(PathAVLnode *arg1,
                                       PathAVLnode *arg2)const{
   return ((T *&)arg1)->GT((T *&)arg2);
   }

template<class T, int size>
inline int TPathAVLtree<T,size>::NEcmp(PathAVLnode *arg1,
                                       PathAVLnode *arg2)const{
   return ((T *&)arg1)->NE((T *&)arg2);
   }

template<class T, int size>
inline void TPathAVLtree<T,size>::DESTR(PathAVLnode *arg1)const{
   delete (T *)arg1;
   }

#endif

