#include <stdlib.h>
#include <time.h>
#include <iostream.h>
#include <iomanip.h>

#include "tpavltre.h"
#include "tpavlnde.h"

/*
 * A simple example program demonstrating the basic
 * functionality of the PathAVLtree.
 *
 * You can additionally control the number of added
 * (and removed) nodes using a command line parameter.
 */

#define StackSize 40

time_t last;

void PrStatus(long int s){
   if(time(NULL)!=last){
      cout << "node number: " << setw(20) << s << '\r' << flush;
	  last=time(NULL);
	  }
   }

int main(int argc, char **argv){
   long int maxNodes=1000, i;
   clock_t start=clock();

   TPathAVLnode<long>                  *nNode;
   TPathAVLnode<long>                  *rNode;
   TPathAVLnode<long>                   wNode(0);
   TPathAVLtree<TPathAVLnode<long>,StackSize>  Tree;
/*
   TPathAVLtree<TPathAVLnode<long>,StackSize>  Tree2;
   TPathAVLtree<TPathAVLnode<long>,StackSize>  Tree3;
*/
   cout << Tree.Version() << endl;

   if(argc==2) maxNodes=atol(argv[1]);

   cout << "\ninserting " << maxNodes << " nodes..." << endl;
   for(i=0;i<maxNodes;i++){
      PrStatus(i);
      nNode=new TPathAVLnode<long> (i);
      Tree.add(nNode);
      }

   cout << "\ndeleting " << maxNodes << " nodes..." << endl;
   for(wNode.Ref()=0;wNode.Ref()<maxNodes;wNode.Ref()++){
      PrStatus(wNode.Ref());
      rNode=Tree.remove(&wNode);
      delete rNode;
      }
   if(Tree.isEmpty()) cout << "deleting was successful."
                           << endl;
   else               cout << "deleting was unsuccessful!"
                           << endl;

   cout << "\ninserting " << maxNodes
        << " nodes (with two-party-steps)..." << endl;
   for(i=0;i<2*maxNodes;i+=2){
      PrStatus(i);
      nNode=new TPathAVLnode<long> (i);
      Tree.add(nNode);
      }

   cout << "\nsearching " << 2*maxNodes << " nodes..." << endl;
   for(wNode.Ref()=0;wNode.Ref()<2*maxNodes;wNode.Ref()++){
      PrStatus(wNode.Ref());
      Tree.search(&wNode);
      }

   cout << "\ndeleting tree at once..." << endl;
   Tree.empty();
   if(Tree.isEmpty()) cout << "deleting was successful."
                           << endl;
   else               cout << "deleting was unsuccessful!"
                           << endl;

   start=clock()-start;

   cout << setfill(' ') << "running time (clock): ";

   if(start>999){
	  maxNodes=1000;
	  while(!(maxNodes%1000)){
		 i=maxNodes;
    	 maxNodes*=1000;
		 }
	  maxNodes=0;
	  while(i){
    	 if(maxNodes || (start/i)>0){
			if(maxNodes) cout << '.' << setw(3) << setfill('0') << (start/i);
	    	else cout << (start/i);
			maxNodes=1;
			start%=i;
	    	}
    	 i/=1000;
    	 }  
      }
   else cout << start;
   
   cout << setfill(' ') << "\n\nyou can use a command line parameter for "
           "assigned numbers of nodes" << endl;
   return 0;
   }
